#include "city_generator.h"
#include "core/mesh_render_entity.h"
#include "core/transform_node.h"

static struct {
    size_t x, y;
} const CITY_SIZE = {20, 20};

#define MODELS_LENGTH 4
static ModelResource models[MODELS_LENGTH] = {
    ResourceEmpty(Model),
    ResourceEmpty(Model),
    ResourceEmpty(Model),
    ResourceEmpty(Model)
};

static void Internal_LoadModelsIfRequired() {
    for(size_t i = 0; i < MODELS_LENGTH; ++i)
        if(models[i].handle == NULL && GetModelResource(TextFormat("building_%c", 'a' + i), &models[i]))
            LoadResource(models[i].handle);
}

static SceneNode *Internal_CreateBuilding(size_t x, size_t y) {
    size_t building_index = (3 * (x + y + GetRandomValue(0, 50))) % MODELS_LENGTH; // semi-random number based on the x,y coordinate
    SceneNode *mesh_render_node = CreateMeshRenderEntity(TextFormat("building_%c", 'a' + building_index));
    SceneNode *transform_node = CreateTransformNode();
    SceneNodeAddChild(transform_node, mesh_render_node);
    Transformable transform = TC_CAST(transform_node->entity, Transformable);
    Transform local = transform.tc->get_transform(transform.data);
    local.translation.x = (float)(x * 40);
    local.translation.z = (float)(y * 40);
    transform.tc->set_transform(transform.data, local);
    return transform_node;
}

SceneNode *GenerateCity() {
    Internal_LoadModelsIfRequired();
        SceneNode *root = CreateTransformNode();
    for(size_t x = 0; x < CITY_SIZE.x; ++x) {
        for(size_t y = 0; y < CITY_SIZE.y; ++y) {
            SceneNodeAddChild(root, Internal_CreateBuilding(x, y));
        }
    }
    return root;
}
