#ifndef PLAYER_CONTROLLER_H
#define PLAYER_CONTROLLER_H

#include "core/scene.h"
#include "core/scene_node_entity.h"
#include "core/transformable.h"
#include "core/renderable.h"
#include "utils/drop.h"
#include "utils/mirror.h"

// Test object created to generally test game features
typedef struct PlayerController {
    SceneNode *node;
    Transformable transform;
    Vector3 fly_input;
    Vector3 rotation;
    float speed;
    bool brake;
    bool boost;
} PlayerController;

SceneNode *CreatePlayerController();
void DestroyPlayerController(PlayerController *self);

extern void PlayerControllerEnterTree(PlayerController *self);
extern void PlayerControllerExitTree(PlayerController *self);
extern void PlayerControllerTick(PlayerController *self, double delta);

extern void PlayerControllerInputRollLeft(PlayerController *self, bool value);
extern void PlayerControllerInputRollRight(PlayerController *self, bool value);
extern void PlayerControllerInputPitchUp(PlayerController *self, bool value);
extern void PlayerControllerInputPitchDown(PlayerController *self, bool value);
extern void PlayerControllerInputYawRight(PlayerController *self, bool value);
extern void PlayerControllerInputYawLeft(PlayerController *self, bool value);
extern void PlayerControllerInputBrake(PlayerController *self, bool value);
extern void PlayerControllerInputBoost(PlayerController *self, bool value);

DECL_REFLECT(PlayerController);
decl_typeclass_impl(SceneNodeEntity, PlayerController);
decl_typeclass_impl(Drop, PlayerController);

#endif // !PLAYER_CONTROLLER_H
